@@ -241,7 +241,7 @@ const long cut, const long conditions, const char sep, const bool doTrim, const
     _HY_HBLCommandHelper.Insert    ((BaseRef)HY_HBL_COMMAND_HARVEST_FREQUENCIES, 
                                     (long)_hyInitCommandExtras (_HY_ValidHBLExpressions.Insert ("HarvestFrequencies(", HY_HBL_COMMAND_HARVEST_FREQUENCIES,false),
                                     -1, 
-                                    "HarvestFrequencies(<receptacle>, <DataSet or DataSetFilter>, <atom INTEGER>, <unit INTEGER <= atom>, <position aware 0 or 1>, [optional site partion], [optional sequence partition] (only for DataSetArguments)",
+                                    "HarvestFrequencies(<receptacle>, <DataSet or DataSetFilter>, <atom INTEGER>, <unit INTEGER <= atom>, <position aware 0 or 1>, [optional site partition], [optional sequence partition] (only for DataSetArguments)",
                                         ',',
                                         true,
                                         false,
@@ -270,7 +270,7 @@ const long cut, const long conditions, const char sep, const bool doTrim, const
     _HY_HBLCommandHelper.Insert    ((BaseRef)HY_HBL_COMMAND_MPI_SEND,
                                     (long)_hyInitCommandExtras (_HY_ValidHBLExpressions.Insert ("MPISend(", HY_HBL_COMMAND_MPI_SEND,false),
                                                                 -1,
-                                                                "MPISend(<node id>, <string | likelihood function ID | filename [in conjuction with argument 3]>, [if specified, treat the second argument as a script path, and use the dict supplied here as input options to the script])",
+                                                                "MPISend(<node id>, <string | likelihood function ID | filename [in conjunction with argument 3]>, [if specified, treat the second argument as a script path, and use the dict supplied here as input options to the script])",
                                                                 ',',
                                                                 true,
                                                                 false,
@@ -280,7 +280,7 @@ const long cut, const long conditions, const char sep, const bool doTrim, const
   _HY_HBLCommandHelper.Insert    ((BaseRef)HY_HBL_COMMAND_CONSTRUCT_CATEGORY_MATRIX,
                                     (long)_hyInitCommandExtras (_HY_ValidHBLExpressions.Insert ("ConstructCategoryMatrix(", HY_HBL_COMMAND_CONSTRUCT_CATEGORY_MATRIX,false),
                                                                 -1,
-                                                                "ConstructCategoryMatrix(<receptacle>, <Likelihood Function|Tree>, [optional <COMPLETE|SHORT|WEIGHTS|CLASSES (default = COMPLETE)> , matrix argument with partitions to include (defaut = all)>])",
+                                                                "ConstructCategoryMatrix(<receptacle>, <Likelihood Function|Tree>, [optional <COMPLETE|SHORT|WEIGHTS|CLASSES (default = COMPLETE)> , matrix argument with partitions to include (default = all)>])",
                                                                 ',',
                                                                 true,
                                                                 false,
@@ -846,7 +846,7 @@ void      _ElementaryCommand::ExecuteCase58 (_ExecutionList& chain)
                 profileDump->MStore ("INSTRUCTION INDEX", instCounter, false);
                 profileDump->MStore ("INSTRUCTION", descList, false);
                 profileDump->MStore ("STATS", execProfile, false);
-                outVar->SetValue (profileDump,false);
+                outVar->SetValue (profileDump,false,true,NULL);
                 chain.doProfile = 0;
                 DeleteObject (chain.profileCounter);
                 chain.profileCounter = nil;
