@@ -41,6 +41,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include "global_object_lists.h"
 #include "avllistxl_iterator.h"
+#include "dataset_filter.h"
 #include "batchlan.h"
 
 //_________________________________________________________
@@ -107,73 +108,69 @@ unsigned long    _DataSetFilter::FindUniqueSequences  (_SimpleList& indices, _Si
     seqs   = theNodeMap.lLength,
     unit   = GetUnitLength();
     
+    _SimpleList _sequenceHashes;
+    _AVLListXL  sequenceHashes     (&_sequenceHashes);
+    
+    auto      check_sequence_match = [&] (long f, long rawSequenceIdx, _SimpleList* & sequencesWithSameScore) -> long {
+        if (f >= 0) {
+            sequencesWithSameScore = (_SimpleList*)sequenceHashes.GetXtra (f);
+            for (long k = 0; k<sequencesWithSameScore->lLength; k++) {
+                bool fit = true;
+                f = sequencesWithSameScore->list_data[k];
+                
+                long fRaw = theNodeMap.list_data[indices.list_data[f]];
+                
+                for (unsigned long site = 0; site < sites && fit; site++){
+                    _Site * thisSite = theData->GetSite(site);
+                    if (thisSite->char_at(fRaw) != thisSite->char_at(rawSequenceIdx)) {
+                        fit = false;
+                        break;
+                    }
+                }
+                
+                if (fit) {
+                    map << f;
+                    counts.list_data[f] ++;
+                    return f;
+                }
+            }
+        }
+        return -1;
+    };
+    
+    auto      add_sequence_match = [&] (_SimpleList* sameScore, long sequenceHash, long sequenceIndex, bool do_map)->void {
+        if (!sameScore) {
+            sameScore = new _SimpleList;
+            sequenceHashes.Insert ((BaseRef)sequenceHash,(long)sameScore,false);
+        }
+        
+        (*sameScore) << indices.lLength;
+        
+        if (do_map) {
+            map     << indices.lLength;
+            indices << sequenceIndex;
+            counts  << 1;
+        }
+    };
+
     if (match_mode == kUniqueMatchExact) { // exact match
-        _SimpleList hashSupport;
-        _AVLListXL  sequenceHashes     (&hashSupport);
         
         for (unsigned long sequenceIndex = 0; sequenceIndex < seqs; sequenceIndex ++){
             _String * thisSequence = GetSequenceCharacters (sequenceIndex);
-            
+  
             long     sequenceHash   = thisSequence->Adler32(),
-            f              = sequenceHashes.Find ((BaseRef)sequenceHash),
+            f              = sequenceHashes.FindLong (sequenceHash),
             rawSequenceIdx = theNodeMap.list_data[sequenceIndex];
             
             DeleteObject (thisSequence);
-            
-            _SimpleList * sameScore = nil;
-            if (f>=0) {
-                sameScore = (_SimpleList*)sequenceHashes.GetXtra (f);
-                for (long k = 0; k<sameScore->lLength; k++) {
-                    bool fit = true;
-                    f = sameScore->list_data[k];
-                    
-                    long fRaw = theNodeMap.list_data[indices.list_data[f]];
-                    
-                    for (unsigned long site = 0; site < sites && fit; site++){
-                        for (unsigned long unitIndex = 0; unitIndex < unit; unitIndex ++){
-                            _Site * thisSite = theData->GetSite(theMap.list_data[unit*site+unitIndex]);
-                            if (thisSite->char_at(fRaw) != thisSite->char_at(rawSequenceIdx)) {
-                                fit = false;
-                                break;
-                            }
-                        }
-                    }
-                    
-                    if (fit) {
-                        /*
-                        StringToConsole (*GetSequenceName(sequenceIndex));
-                        BufferToConsole("==");
-                        StringToConsole (*GetSequenceName(f));
-                        NLToConsole();
-                        
-                        StringToConsole (*GetSequenceCharacters(sequenceIndex));
-                        NLToConsole();
-                        StringToConsole (*GetSequenceCharacters(f));
-                        NLToConsole();
-                        NLToConsole();
-                        */
 
-                        map << f;
-                        counts.list_data[f] ++;
-                        
-                    } else {
-                        f = -1;
-                    }
-                }
-            }
+            _SimpleList * sameScore = nil;
+            f = check_sequence_match (f, rawSequenceIdx, sameScore);
+            
             if (f==-1) { // fit failed or unique site
-                if (!sameScore) {
-                    sameScore = new _SimpleList;
-                    sequenceHashes.Insert ((BaseRef)sequenceHash,(long)sameScore,false);
-                }
-                
-                (*sameScore) << indices.lLength;
-                map     << indices.lLength;
-                indices << sequenceIndex;
-                counts  << 1;
+                add_sequence_match (sameScore, sequenceHash, sequenceIndex, true);
             }
         }
-        
     }
     else{
         long             vd  = GetDimension(true);
@@ -181,13 +178,37 @@ unsigned long    _DataSetFilter::FindUniqueSequences  (_SimpleList& indices, _Si
         hyFloat      *translatedVector  = new hyFloat [vd],
                      *translatedVector2 = new hyFloat [vd];
         
-        _String         state1 (unit), state2 (unit);
+        _String      state1 (unit), state2 (unit);
+        
+        long         *referenceStates = (long*) new long[sites];
         
         sites = sites / unit;
         
         for (long sequenceIndex = 0; sequenceIndex < seqs; sequenceIndex++) {
             bool check_state = false;
             
+            //printf ("%ld (%ld)\n", sequenceIndex, indices.countitems());
+            
+            _String * thisSequence = GetSequenceCharacters (sequenceIndex);
+            
+            long     sequenceHash   = thisSequence->Adler32(),
+                     f              = sequenceHashes.FindLong (sequenceHash),
+                     rawSequenceIdx = theNodeMap.list_data[sequenceIndex];
+            
+            DeleteObject(thisSequence);
+            _SimpleList * sameScore = nil;
+            f = check_sequence_match (f, rawSequenceIdx, sameScore);
+            
+            if (f >= 0L) {
+                //printf ("Adler Hash match for %ld (%ld/%ld)\n", sequenceIndex, f, indices.countitems());
+                continue;
+            }
+            
+            for (long m=0L; m<sites; m++) {
+                RetrieveState (m,sequenceIndex,    state1,false);
+                referenceStates[m] = Translate2Frequencies (state1, translatedVector,  true);
+            }
+            
             for (long idx=0L; idx<indices.countitems(); idx++) {
                 check_state = true;
                 for (long m=0L; m<sites; m++) {
@@ -195,100 +216,92 @@ unsigned long    _DataSetFilter::FindUniqueSequences  (_SimpleList& indices, _Si
                     RetrieveState (m,sequenceIndex,    state1,false);
                     RetrieveState (m,indices.get(idx), state2,false);
                     
-                    long idx1 = Translate2Frequencies (state1, translatedVector,  true),
-                         idx2 = Translate2Frequencies (state2, translatedVector2, true);
-                    
-                    /*if (idx == 85 && sequenceIndex == 88) {
-                        printf ("(%ld, %ld) %ld = %ld %ld\n", sequenceIndex, indices.list_data[idx], m, idx1, idx2);
-                    }*/
-                    
-                    if (idx2 >=0 && idx1 >=0) { // fully resolved
-                        if (idx1==idx2) {
-                            continue;
-                        } else {
-                            check_state = false;
-                            break;
-                        }
-                    } else {
+                    if (state1 != state2) { // only check different states
+                        long idx1 = referenceStates[m],//Translate2Frequencies (state1, translatedVector,  true),
+                             idx2 = Translate2Frequencies (state2, translatedVector2, true);
                         
-                        // check for equal ambigs
-                        long k = 0L;
-                        for (; k < vd; k++){
-                            if (translatedVector[k] != translatedVector2[k]){
-                                break;
-                            }
-                        }
+                        /*if (idx == 85 && sequenceIndex == 88) {
+                            printf ("(%ld, %ld) %ld = %ld %ld\n", sequenceIndex, indices.list_data[idx], m, idx1, idx2);
+                        }*/
                         
-                        if (k == vd) { // resolutions matched -- this is always OK
-                            continue;
-                        }
-                        
-                        
-                        if (match_mode == kUniqueMatchExactOrGap) {
-                            
-                            long count1 = 0L,
-                                 count2 = 0L;
-                            
-                            for (long t = 0L; t<vd; t++) {
-                                count1 += translatedVector[t]  > 0.0;
-                                count2 += translatedVector2[t] > 0.0;
-                            }
-                            
-                            // Gaps can match resolved positions, but not the other way around
-                            // count2 is the number of resolutions in the currently stored sequence
-                            if (count2 > 1L && count2 < vd || count1 < vd) {
+                        if (idx2 >=0 && idx1 >=0) { // fully resolved
+                            if (idx1==idx2) {
+                                continue;
+                            } else {
                                 check_state = false;
                                 break;
                             }
-                            
-                            
                         } else {
-                            bool first  = (match_mode == kUniqueMatchSuperset),
-                                 second = first;
+                            //RetrieveState (m,sequenceIndex,    state1,false);
+                            Translate2Frequencies (state1, translatedVector,  true);
+                            // check for equal ambigs
+                            long k = 0L;
+                            for (; k < vd; k++){
+                                if (translatedVector[k] != translatedVector2[k]){
+                                    break;
+                                }
+                            }
                             
-                            if (match_mode == kUniqueMatchSuperset){
-                                for (long t = 0L; (first||second)&&(t<vd); t++) {
-                                    if (translatedVector[t]>0.0) {
-                                        second &= (translatedVector2[t]>0.0);
-                                    }
-                                    if (translatedVector2[t]>0.0) {
-                                        first  &= (translatedVector[t]>0.0);
-                                    }
-                                }
-                                if (!(first||second)) {
-                                    check_state = false;
-                                    break;
-                                }
-                            } else {
+                            if (k == vd) { // resolutions matched -- this is always OK
+                                continue;
+                            }
+                            
+                            
+                            if (match_mode == kUniqueMatchExactOrGap) {
+                                
+                                long count1 = 0L,
+                                     count2 = 0L;
+                                
                                 for (long t = 0L; t<vd; t++) {
-                                    if (translatedVector[t]>0.0) {
-                                        second |= (translatedVector2[t]>0.0);
-                                    }
-                                    if (translatedVector2[t]>0.0) {
-                                        first  |= (translatedVector[t]>0.0);
-                                    }
+                                    count1 += translatedVector[t]  > 0.0;
+                                    count2 += translatedVector2[t] > 0.0;
                                 }
-                                if (!(first&&second)) {
+                                
+                                // Gaps can match resolved positions, but not the other way around
+                                // count2 is the number of resolutions in the currently stored sequence
+                                if (count2 > 1L && count2 < vd || count1 < vd) {
                                     check_state = false;
                                     break;
                                 }
+                                
+                                
+                            } else {
+                                bool first  = (match_mode == kUniqueMatchSuperset),
+                                     second = first;
+                                
+                                if (match_mode == kUniqueMatchSuperset){
+                                    for (long t = 0L; (first||second)&&(t<vd); t++) {
+                                        if (translatedVector[t]>0.0) {
+                                            second &= (translatedVector2[t]>0.0);
+                                        }
+                                        if (translatedVector2[t]>0.0) {
+                                            first  &= (translatedVector[t]>0.0);
+                                        }
+                                    }
+                                    if (!(first||second)) {
+                                        check_state = false;
+                                        break;
+                                    }
+                                } else {
+                                    for (long t = 0L; t<vd; t++) {
+                                        if (translatedVector[t]>0.0) {
+                                            second |= (translatedVector2[t]>0.0);
+                                        }
+                                        if (translatedVector2[t]>0.0) {
+                                            first  |= (translatedVector[t]>0.0);
+                                        }
+                                    }
+                                    if (!(first&&second)) {
+                                        check_state = false;
+                                        break;
+                                    }
+                                }
                             }
                         }
                     }
                 }
-                
+            
                 if (check_state) {
-                    /*StringToConsole (*GetSequenceName(sequenceIndex));
-                    BufferToConsole("== [ref]");
-                    StringToConsole (*GetSequenceName(indices.get (idx)));
-                    NLToConsole();
-                    
-                    StringToConsole (*GetSequenceCharacters(sequenceIndex));
-                    NLToConsole();
-                    StringToConsole (*GetSequenceCharacters(indices.get (idx)));
-                    NLToConsole();
-                    NLToConsole();*/
-
                     map << idx;
                     counts.list_data[idx] ++;
                     break;
@@ -296,6 +309,7 @@ unsigned long    _DataSetFilter::FindUniqueSequences  (_SimpleList& indices, _Si
             }
             
             if (!check_state){
+                add_sequence_match (sameScore, sequenceHash, sequenceIndex, false);
                 map     << indices.lLength;
                 indices << sequenceIndex;
                 counts  << 1;
@@ -305,6 +319,7 @@ unsigned long    _DataSetFilter::FindUniqueSequences  (_SimpleList& indices, _Si
         
         delete [] translatedVector;
         delete [] translatedVector2;
+        delete [] referenceStates;
     }
     
     
@@ -811,17 +826,6 @@ _String* _DataSetFilter::MakeSiteBuffer (void) const {
     return new _String ((unsigned long)unitLength);
 }
 
-//_______________________________________________________________________
-void _DataSetFilter::retrieve_individual_site_from_raw_coordinates (_String& store, unsigned long site, unsigned long sequence) const {
-  if (unitLength==1UL) {
-    store.set_char (0, (((_String**)theData->list_data)[theData->theMap.list_data[theMap.list_data[site]]])->char_at (sequence));
-  } else {
-    site*=unitLength;
-    for (unsigned long k = 0UL; k<unitLength; k++) {
-      store.set_char (k, ((_String**)theData->list_data)[theData->theMap.list_data[theMap.list_data[site++]]]->char_at(sequence));
-    }
-  }
-}
 
 //_______________________________________________________________________
 
@@ -867,22 +871,17 @@ _List *  _DataSetFilter::ComputePatternToSiteMap (void) const {
 }
 
 
+
 //_______________________________________________________________________
-
-char _DataSetFilter::direct_index_character (unsigned long site, unsigned long sequence) const {
-  return (((_String**)theData->list_data)[theData->theMap.list_data[theMap.list_data[site]]])->char_at(sequence);
-}
-
-  //_______________________________________________________________________
-
+ 
 bool     _DataSetFilter::CompareTwoSites (unsigned long site1, unsigned long site2, unsigned long pos1) const {
   pos1 = theNodeMap.list_data[pos1];
   
-  
   switch (unitLength) {
+          
     case 3: { // codon
-      site1*=3;
-      site2*=3;
+      site1 = (site1 << 1) + site1;
+      site2 = (site2 << 1) + site2;
       return direct_index_character (site1, pos1)     == direct_index_character (site2,     pos1) &&
       direct_index_character (site1 + 1, pos1) == direct_index_character (site2 + 1, pos1) &&
       direct_index_character (site1 + 2, pos1) == direct_index_character (site2 + 2, pos1);
@@ -893,15 +892,15 @@ bool     _DataSetFilter::CompareTwoSites (unsigned long site1, unsigned long sit
     }
       
     case 2: { // di-nucs
-      site1*=2;
-      site2*=2;
+      site1 = site1 << 1;
+      site2 = site1 << 1;
       return direct_index_character (site1, pos1)     == direct_index_character (site2,     pos1) &&
       direct_index_character (site1 + 1, pos1) == direct_index_character (site2 + 1, pos1);
     }
       
       
     default: {
-      site1*=unitLength;
+      site1*=unitLength;    
       site2*=unitLength;
       
       
@@ -1020,14 +1019,19 @@ long    _DataSetFilter::HasExclusions (unsigned long site, _SimpleList* theExc,
     if (theNodeMap.countitems()) {
         _String buffer ((unsigned long)GetUnitLength());
   
+        unsigned long   filter_dim = GetDimension(false);
+
         for (unsigned long k = 0UL; k<theNodeMap.countitems(); k++) {
             RetrieveState           (site, k, buffer, false);
-            Translate2Frequencies   (buffer, store, false, false);
+            long idx =              Translate2Frequencies   (buffer, store, false, false);
             
-            unsigned long   filter_dim = GetDimension(false);
             long            found_forbidden = -1;
-          
-            for (unsigned long character = 0UL ;  character < filter_dim;  character ++) {
+            if (idx >= 0L) {
+                if (theExc->Find(idx) != kNotFound) {
+                    found_forbidden = k;
+                }
+            } else {
+                for (unsigned long character = 0UL ;  character < filter_dim;  character ++) {
                    if (store[character] > 0.0) {
                       if (theExc->Find(character) == kNotFound) {
                           found_forbidden = -1;
@@ -1036,6 +1040,7 @@ long    _DataSetFilter::HasExclusions (unsigned long site, _SimpleList* theExc,
                         found_forbidden = k;
                       }
                   }
+                }
             }
           
             if (found_forbidden >= 0L) {
@@ -1061,13 +1066,15 @@ _Matrix* _DataSetFilter::ComputePairwiseDifferences (long i, long j, _hy_dataset
         
         _Matrix     *res   = new _Matrix  (mxDim,mxDim,false,true);
         
-        hyFloat  *sm1   = new hyFloat[mxDim],
-        *sm2   = new hyFloat[mxDim];
+        hyFloat  *sm1   = (hyFloat*)alloca (sizeof(hyFloat)*mxDim),
+                 *sm2   = (hyFloat*)alloca (sizeof(hyFloat)*mxDim);
         
         
+        char    *buffer1 = (char*)alloca (sizeof (char) * (1+unitLength)),
+                *buffer2 = (char*)alloca (sizeof (char) * (1+unitLength));
         
-        _String      state1 ((unsigned long)unitLength),
-                     state2 ((unsigned long)unitLength);
+        _String      state1 ((unsigned long)unitLength, buffer1),
+                     state2 ((unsigned long)unitLength, buffer2);
         
         
         if (conversionCache.lLength == 0) {
@@ -1163,12 +1170,12 @@ _Matrix* _DataSetFilter::ComputePairwiseDifferences (long i, long j, _hy_dataset
                             long k = 0,
                             u = GetDimension (false);
                             
-                            for (long i = 0; i<u; i++) {
-                                if (i==theExclusions.list_data[k] && k<theExclusions.lLength) {
+                            for (long idx = 0; idx<u; idx++) {
+                                if (idx==theExclusions.list_data[k] && k<theExclusions.lLength) {
                                     k++;
                                     continue;
                                 }
-                                freqsAtSite->theData[i-k] = freqsAtSite->theData[i];
+                                freqsAtSite->theData[idx-k] = freqsAtSite->theData[idx];
                             }
                         }
                         //XferwCorrection (freqsAtSite->theData, freqsAtSite->theData, mxDim);
@@ -1394,12 +1401,10 @@ _Matrix* _DataSetFilter::ComputePairwiseDifferences (long i, long j, _hy_dataset
             }
         }
         
-        delete[] sm1;
-        delete[] sm2;
         
         return res;
     }
-    catch (const _String error) {
+    catch (const _String& error) {
         HandleApplicationError(error);
         return    new _Matrix (1,1,false,true);
         
@@ -1620,13 +1625,16 @@ long    _DataSetFilter::Translate2Frequencies (_String const& str, hyFloat* parv
     long mapped_resolution_count = correct_for_exclusions && theExclusions.nonempty() ? theExclusions.CorrectForExclusions(store, resolution_count) : resolution_count;
     
     /* handle the cases when no unambiguous resolutions were available */
+
+    if (mapped_resolution_count == 1L) {
+        parvect[store[0]] = 1.;
+        return store[0];
+    }
+    
     for (long i = 0L; i < mapped_resolution_count; i++) {
         parvect[store[i]] = 1.;
     }
     
-    if (mapped_resolution_count == 1L) {
-        return store[0];
-    }
     if (mapped_resolution_count == 0L && resolution_count == 0L && smear) {
         InitializeArray(parvect, dimension, 1.);
     }
@@ -1894,8 +1902,27 @@ void    _DataSetFilter::ConvertCodeToLettersBuffered (long code, unsigned char u
 
 void    _DataSetFilter::internalToStr (FILE * file ,_StringBuffer * string_buffer) {
   
+    //    case 4: // labels, sequential
+    //case 5: // labels, interleaved
+    //case 6: // no labels, sequential
+    //case 7: { // no labels, interleaved
+
+  const enum    {
+      kFormatMEGASequential             = 0,
+      kFormatMEGAInterleaved            = 1,
+      kFormatPHYLIPSequential           = 2,
+      kFormatPHYLIPInterleaved          = 3,
+      kFormatNEXUSLabelsSequential      = 4,
+      kFormatNEXUSLabelsInterleaved     = 5,
+      kFormatNEXUSSequential            = 6,
+      kFormatNEXUSInterleaved           = 7,
+      kFormatCharacterList              = 8,
+      kFormatFASTASequential            = 9,
+      kFormatFASTAInterleaved           = 10,
+      kFormatPAML                       = 11
+  } datafile_format = kFormatMEGASequential;
   
-  auto trim_to_10 = [] (const _String& seq_name) -> _String const& {
+  auto trim_to_10 = [] (const _String& seq_name) -> _String const {
     if (seq_name.length() >= 10) {
       return seq_name.Cut (0,9) & ' ';
     }
@@ -1906,6 +1933,7 @@ void    _DataSetFilter::internalToStr (FILE * file ,_StringBuffer * string_buffe
        printWidth   = hy_env::EnvVariableGetNumber(hy_env::data_file_default_width),
        gapWidth     = hy_env::EnvVariableGetNumber(hy_env::data_file_gap_width);
   
+
     // write out the file with this dataset filter
   
   unsigned long sequence_count = NumberSpecies(),
@@ -1942,13 +1970,13 @@ void    _DataSetFilter::internalToStr (FILE * file ,_StringBuffer * string_buffe
   }
   
   switch (outputFormat) {
-    case 1: // hash-mark interleaved
-    case 10: { // FASTA interleaved
+    case kFormatMEGAInterleaved: // hash-mark interleaved
+    case kFormatFASTAInterleaved: { // FASTA interleaved
       
       long sitesDone    = 0,
       upTo;
       
-      char seqDelimiter = (outputFormat==1)?'#':'>';
+      char seqDelimiter = (outputFormat==kFormatMEGAInterleaved)?'#':'>';
       
       for (unsigned long i = 0UL; i<theNodeMap.lLength; i++) {
         write_here << seqDelimiter
@@ -1983,8 +2011,8 @@ void    _DataSetFilter::internalToStr (FILE * file ,_StringBuffer * string_buffe
       break;
     }
       
-    case 2:     // PHYLIP sequential
-    case 11:    // PAML
+    case kFormatPHYLIPSequential:     // PHYLIP sequential
+    case kFormatPAML:    // PAML
     {
       
       write_here << _String((long)theNodeMap.lLength)
@@ -1997,7 +2025,7 @@ void    _DataSetFilter::internalToStr (FILE * file ,_StringBuffer * string_buffe
         _String const * sequence_name = GetSequenceName(i);
         _String sequence_name_10;
         
-        if (outputFormat == 2) { // PHYLIP
+        if (outputFormat == kFormatPHYLIPSequential) { // PHYLIP
           sequence_name_10 = trim_to_10 (*sequence_name);
         } else {
           sequence_name_10 = *sequence_name & "  ";
@@ -2021,7 +2049,7 @@ void    _DataSetFilter::internalToStr (FILE * file ,_StringBuffer * string_buffe
       break;
     }
       
-    case 3: { // phylip interleaved
+    case kFormatPHYLIPInterleaved: { // phylip interleaved
               // print PHYLIP format header
               //fprintf (dest,"$FORMAT:\"PHYLIPI\"\n");
               // print number of species and sites
@@ -2072,10 +2100,10 @@ void    _DataSetFilter::internalToStr (FILE * file ,_StringBuffer * string_buffe
       
         // various flavors of NEXUS
       
-    case 4: // labels, sequential
-    case 5: // labels, interleaved
-    case 6: // no labels, sequential
-    case 7: { // no labels, interleaved
+    case kFormatNEXUSLabelsSequential: // labels, sequential
+    case kFormatNEXUSLabelsInterleaved: // labels, interleaved
+    case kFormatNEXUSSequential: // no labels, sequential
+    case kFormatNEXUSInterleaved: { // no labels, interleaved
               // write out the header
       
       write_here << "#NEXUS\n\nBEGIN TAXA;\n\tDIMENSIONS NTAX = "
@@ -2124,10 +2152,10 @@ void    _DataSetFilter::internalToStr (FILE * file ,_StringBuffer * string_buffe
       if (theData->theTT->GetSkipChar()) {
         write_here << "\n\t\tMISSING=" << theData->theTT->GetSkipChar();
       }
-      if (outputFormat>5) {
+      if (outputFormat == kFormatNEXUSInterleaved || outputFormat == kFormatNEXUSSequential) {
         write_here << "\n\t\tNOLABELS";
       }
-      if (outputFormat%2) {
+      if (outputFormat == kFormatNEXUSInterleaved || outputFormat == kFormatNEXUSLabelsInterleaved) {
         write_here << "\n\t\tINTERLEAVE";
       }
       
@@ -2151,9 +2179,9 @@ void    _DataSetFilter::internalToStr (FILE * file ,_StringBuffer * string_buffe
       }
       
       
-      if (outputFormat%2==0) { // sequential
+      if (outputFormat == kFormatNEXUSLabelsSequential || outputFormat == kFormatNEXUSSequential) { // sequential
         for (unsigned long i=0UL; i< sequence_count; i++) {
-          if (outputFormat == 4) { // labels
+          if (outputFormat == kFormatNEXUSLabelsSequential) { // labels
             write_here << "\n\t'"
             << GetSequenceName(i)
             << '\''
@@ -2180,7 +2208,7 @@ void    _DataSetFilter::internalToStr (FILE * file ,_StringBuffer * string_buffe
           
           
           for (unsigned long i=0UL; i< sequence_count; i++) {
-            if (outputFormat == 5) { // labels
+            if (outputFormat == kFormatNEXUSLabelsInterleaved) { // labels
               write_here << "\n\t'"
               << GetSequenceName(i)
               << '"'
@@ -2204,7 +2232,7 @@ void    _DataSetFilter::internalToStr (FILE * file ,_StringBuffer * string_buffe
       break;
     }
       
-    case 8: {
+    case kFormatCharacterList: {
       for (unsigned long i = 0UL; i< sequence_count; i++) {
         write_here << (*theData)(theOriginalOrder(0),theNodeMap(i),1);
         for (unsigned long j = 1UL; j<site_count; j++) {
@@ -2216,7 +2244,7 @@ void    _DataSetFilter::internalToStr (FILE * file ,_StringBuffer * string_buffe
     }
       
     default: { // hash-mark sequential
-      char seqDelimiter = (outputFormat==9)?'>':'#';
+      char seqDelimiter = (outputFormat==kFormatFASTASequential)?'>':'#';
       
       for (unsigned long i = 0UL; i< sequence_count; i++) {
         write_here << seqDelimiter << GetSequenceName(i);
@@ -2234,23 +2262,24 @@ void    _DataSetFilter::internalToStr (FILE * file ,_StringBuffer * string_buffe
     }
   }
   
-  if (outputFormat != 8) {
+  if (outputFormat != kFormatCharacterList) {
       if (hy_env::EnvVariableTrue(hy_env::data_file_tree)) {
           HBLObjectRef tree_var = hy_env::EnvVariableGet(hy_env::data_file_tree_string, HY_ANY_OBJECT);
           if (tree_var) {
             _String* treeString = (_String*)(tree_var->Compute())->toStr();
             switch (outputFormat) {
-              case 0:
-              case 1:
-              case 9:
-              case 10: {
+              case kFormatMEGASequential:
+              case kFormatMEGAInterleaved:
+              case kFormatFASTASequential:
+              case kFormatFASTAInterleaved:
+              case kFormatPAML: {
                 write_here << kStringFileWrapperNewLine
                 << kStringFileWrapperNewLine
                 << *treeString;
                 break;
               }
-              case 2:
-              case 3: {
+              case kFormatPHYLIPSequential:
+              case kFormatPHYLIPInterleaved: {
                 write_here << "\n1\n" << *treeString;
                 break;
               }
