@@ -57,8 +57,8 @@
 #include <time.h>
 #include <float.h>
 #include <signal.h>
-#include <stdlib.h>
 
+//#define __HYPHY_MPI_MESSAGE_LOGGING__
 
 using     namespace hy_env;
 
@@ -91,15 +91,16 @@ namespace hy_global {
                      terminate_execution = false,
                      has_terminal_stdout = true,
                      has_terminal_stderr = true,
-                     ignore_kw_defaults  = false;
+                     ignore_kw_defaults  = false,
+                     force_verbosity_from_cli = false;
     
-    FILE            *hy_error_log_file,
-                    *hy_message_log_file;
+    FILE            *hy_error_log_file = NULL,
+                    *hy_message_log_file = NULL;
     
     hyTreeDefinitionPhase
                      isDefiningATree = kTreeNotBeingDefined;
     
-    hyFloat          kMachineEpsilon = 2.*DBL_EPSILON;
+    const hyFloat    kMachineEpsilon = 2.*DBL_EPSILON;
 
     
     _String const    kEmptyString,
@@ -119,11 +120,12 @@ namespace hy_global {
                      kErrorStringDatasetRefIndexError ("Dataset index reference out of range"),
                      kErrorStringMatrixExportError    ("Export matrix called with a non-polynomial matrix argument"),
                      kErrorStringNullOperand          ("Attempting to operate on an undefined value; this is probably the result of an earlier 'soft' error condition"),
-                      kHyPhyVersion  = _String ("2.5.1"),
+                     kHyPhyVersion  = _String ("2.5.20"),
     
                     kNoneToken = "None",
                     kNullToken = "null",
-                    kNoKWMatch = "__input_value_not_given__";
+                    kNoKWMatch = "__input_value_not_given__",
+                    kEndIteration = "__iterator_end__loop__";
   
     _String
                      hy_base_directory,
@@ -132,7 +134,7 @@ namespace hy_global {
                      hy_standard_library_directory ("TemplateBatchFiles"),
                      hy_standard_model_directory   ("TemplateModels"),
                      hy_error_log_name             ("errors.log"),
-                     hy_messages_log_name          ("messages.log");
+                     hy_messages_log_name          ; // 20200222 : DISABLE MESSAGES BY DEFAULT
   
   
     _List            _hy_application_globals_aux,
@@ -309,7 +311,6 @@ namespace hy_global {
         
         using namespace hyphy_global_objects;
         
-        ClearBFFunctionLists();
         executionStack.Clear();
         loadedLibraryPaths.Clear(true);
         _HY_HBL_Namespaces.Clear();
@@ -319,7 +320,7 @@ namespace hy_global {
             likeFuncNamesList.Clear();
             dataSetList.Clear();
             dataSetNamesList.Clear();
-            batchLanguageFunctions.Clear();
+            ClearBFFunctionLists();
             compiledFormulaeParameters.Clear();
             modelNames.Clear();
             KillExplicitModelFormulae ();
@@ -394,6 +395,7 @@ namespace hy_global {
             for (long file_index = 0; file_index < 2; file_index++) {
                 long                    p   = 1L;
                 
+                if (prefix[file_index]->empty()) continue;
     #ifndef __HYPHYMPI__
                 _String file_name (*prefix[file_index]);
         #if defined  __MINGW32__
@@ -423,6 +425,9 @@ namespace hy_global {
         }
 #endif
         
+        if (hy_env::cli_env_settings.nonempty()) {
+            _ExecutionList (hy_env::cli_env_settings).Execute();
+        }
         return hy_error_log_file && hy_message_log_file;
     }
     
@@ -794,16 +799,19 @@ namespace hy_global {
     time (&c_time);
     
     if (do_gmt) {
-      tm* gmt = gmtime (&c_time);
+      tm  gmt;
+      gmtime_r (&c_time, &gmt);
       
-      return _StringBuffer (_String((long)1900+gmt->tm_year)) << '/' << _String (1+(long)gmt->tm_mon) << '/'
-             << _String ((long)gmt->tm_mday) << ' ' << _String ((long)gmt->tm_hour) << ':' & _String ((long)gmt->tm_min);
+      return _StringBuffer (_String((long)1900+gmt.tm_year)) << '/' << _String (1+(long)gmt.tm_mon) << '/'
+            << _String ((long)gmt.tm_mday) << ' ' << _String ((long)gmt.tm_hour) << ':' & _String ((long)gmt.tm_min);
     }
     
-    tm*     local_time = localtime (&c_time);
-    
-    return  asctime (local_time);
+    tm    local_time;
+    localtime_r (&c_time, &local_time);
+    char  time_buffer [128];
     
+    asctime_r (&local_time, time_buffer);
+    return time_buffer;
   }
   
   //____________________________________________________________________________________
@@ -820,7 +828,7 @@ namespace hy_global {
           char tmpFileName[] = "/tmp/HYPHY-XXXXXX";
           int fileDescriptor = mkstemp(tmpFileName);
           if (fileDescriptor == -1){
-            throw ("Failed to create a temporary file name");
+            throw _String("Failed to create a temporary file name");
           }
           path_name = tmpFileName;
           CheckReceptacleAndStore(&hy_env::last_file_path,kEmptyString,false, new _FString (path_name, false), false);
