@@ -138,12 +138,18 @@ unsigned long        _Variable::ObjectClass (void) const {
 }
 
 //__________________________________________________________________________________
-bool _Variable::CheckFForDependence (long idx, bool opt)
-{
+bool _Variable::CheckFForDependence (long idx, bool opt) {
     if (varFormula) {
         return varFormula->CheckFForDependence (idx, opt);
     }
+    return false;
+}
 
+//__________________________________________________________________________________
+bool _Variable::CheckFForDependence (_AVLList const& indices, bool opt) {
+    if (varFormula) {
+        return varFormula->CheckFForDependence (indices, opt);
+    }
     return false;
 }
 
@@ -200,7 +206,7 @@ _Variable::_Variable (_String const&s, _String const &f, bool isG) {
     if (varFormula->IsAConstant()) {
         HBLObjectRef theP = varFormula->Compute();
         if (theP) {
-            SetValue (theP);
+            SetValue (theP,true,true,NULL);
             delete   (varFormula);
             varFormula = nil;
         } else {
@@ -252,8 +258,8 @@ HBLObjectRef  _Variable::ComputeMatchingType(long type) {
 
 //__________________________________________________________________________________
 
-HBLObjectRef  _Variable::Compute (void) // compute or return the value
-{
+HBLObjectRef  _Variable::Compute (void) {
+    // compute or return the value
     // call_count++;
     
     auto update_var_value = [this] () -> void {
@@ -332,76 +338,121 @@ void  _Variable::CompileListOfDependents (_SimpleList& rec)
 //__________________________________________________________________________________
 void  _Variable::SetValue (hyFloat new_value) {
 // set the value of the var
-  this->SetValue (new _Constant (new_value), false);
+  this->SetValue (new _Constant (new_value), false,true,NULL);
 }
 
 //__________________________________________________________________________________
-void  _Variable::SetValue (HBLObjectRef theP, bool dup) // set the value of the var
-{
+void        SetVariablesToOwnValues (_AVLList const & indices) {
+    /* compute current variable values*/
+    _SimpleList  hist;
+    long         ls = -1L, cn;
+    
+    cn = indices.Traverser (hist,ls,indices.GetRoot());
+    
+    _SimpleList _index2value;
+    _AVLList    index2value (&_index2value);
+    
+    while (cn>=0) {
+        _Variable* this_var = LocateVar (indices.RetrieveLong(cn));
+        if (this_var->IsIndependent()) {
+            // do nothing
+        } else {
+            HBLObjectRef curValue = this_var->Compute();
+            curValue->AddAReference();
+            index2value.InsertNumber(this_var->get_index());
+            this_var->SetValue(curValue, false, false, NULL);
+            //printf ("SetVariablesToOwnValues : Set own value for %s\n", this_var->GetName()->get_str());
+        }
+        cn = indices.Traverser (hist,ls);
+    }
+    
+    // now clean up variable containers that may have had these as dependent template variables
+    
+    DoForEachVariable([&index2value] (_Variable* v, long v_idx) -> void {
+        if (v->IsContainer()) {
+            ((_VariableContainer*)v)->RemoveDependance (index2value);
+        }
+    });
+    
+    DoForEachLikelihoodFunction ([&index2value] (_LikelihoodFunction *lf, long idx) -> void {
+        lf->UpdateDependent(index2value);
+    });
+    
+    // update likelihood functions that may have had these as dependent variables
+    
+}
+
+//__________________________________________________________________________________
+void  _Variable::SetValue (HBLObjectRef theP, bool dup, bool do_checks, _AVLList* keep_track_of_changes) { // set the value of the var
     //hasBeenChanged = true;
     if (varFlags & HY_VARIABLE_COMPUTING) {
-        HandleApplicationError (_String ("A recursive dependency error in _Variable::SetValue; this is an HBL implementation bug; offending variable is '") & *GetName() & "'");
+        HandleApplicationError (_String ("A recursive dependency error in _Variable::SetValue; this is an HBL implementation bug; offending variable is ") & GetName()->Enquote());
         return ;
     }
   
     varFlags &= HY_VARIABLE_SET;
     varFlags |= HY_VARIABLE_CHANGED;
+    
+    if (keep_track_of_changes) {
+        keep_track_of_changes->InsertNumber (get_index());
+    }
 
     long     valueClass = theP->ObjectClass();
   
-    /*bool     doPrint = (*theName) == _String("_pattern_info_");
-    if (doPrint) {
-      printf ("Setting %s to %s\n", theName->get_str(), _String((_String*)theP->toStr()).get_str());
-    }*/
-
     if (valueClass==NUMBER) {
       
-      
         if (varFormula) {
 
-            // also update the fact that this variable is no longer dependent in all declared
-            // variable containers which contain references to this variable
-            
-            for (unsigned long i = 0UL; i<variablePtrs.lLength; i++) {
-                if (freeSlots.Find(i)>=0) {
-                    continue;
-                }
-                _Variable* theV = (_Variable*)variablePtrs(i);
-                if (theV->IsContainer()) {
-                    _VariableContainer* theVC = (_VariableContainer*)theV;
-                    if (!theVC->RemoveDependance (theIndex)) {
-                        ReportWarning ((_String("Can't make variable ")&*GetName()&" independent in the context of "&*theVC->GetName()&" because its template variable is not independent."));
-                        continue;
+            if (do_checks) {
+                // also update the fact that this variable is no longer dependent in all declared
+                // variable containers which contain references to this variable
+                
+                DoForEachVariable([this] (_Variable* v, long v_idx) -> void {
+                    if (v->IsContainer()) {
+                        if (!((_VariableContainer*)v)->RemoveDependance (this->theIndex)) {
+                            ReportWarning ((_String("Can't make variable ")&GetName()->Enquote()&" independent in the context of "&*v->GetName()&" because its template variable is not independent."));
+                        }
+                    }
+                });
+                
+                for (unsigned long i = 0UL; i<likeFuncList.lLength; i++)
+                    if (((_String*)likeFuncNamesList(i))->nonempty()) {
+                        ((_LikelihoodFunction*)likeFuncList(i))->UpdateDependent(theIndex);
                     }
-                }
             }
-            for (unsigned long i = 0UL; i<likeFuncList.lLength; i++)
-                if (((_String*)likeFuncNamesList(i))->nonempty()) {
-                    ((_LikelihoodFunction*)likeFuncList(i))->UpdateDependent(theIndex);
-                }
 
             //_Formula::Clear();
             delete varFormula;
             varFormula = nil;
         }
-        if (varValue) {
-            DeleteObject (varValue);
-            varValue=nil;
-        }
-
+        
         theValue = theP->Value();
-
-        if (!dup) {
-            DeleteObject (theP);
-        }
-
+        
         if (theValue<lowerBound || theValue>upperBound) {
+            /*if (verbosity_level >= 100) {
+                printf ("\n<========== OUT OF BOUNDS FOR VARIABLE %s : value %.16g, range [%.16g, %.16g] =========>\n" , theName->get_str(), theValue, lowerBound, upperBound);
+            }*/
             if (theValue <= lowerBound+1e-50) {
                 theValue = lowerBound;
             } else {
                 theValue = upperBound;
             }
         }
+        
+        if (varValue) {
+            if (varValue->ObjectClass() == NUMBER && varValue->SingleReference()) {
+                ((_Constant*)varValue)->SetValue(theValue);
+            } else {
+                DeleteAndZeroObject (varValue);
+            }
+        }
+
+
+        if (!dup) {
+            DeleteObject (theP);
+        }
+
+        
     } else {
         if (varFormula) {
             /*if (doPrint) {
@@ -411,20 +462,35 @@ void  _Variable::SetValue (HBLObjectRef theP, bool dup) // set the value of the
             varFormula = nil;
             //theFormula.Clear();
         }
+        
+        
         if (varValue) {
+            if (dup && varValue->SingleReference() && varValue->ObjectClass() == theP->ObjectClass()) {
+                switch (varValue->ObjectClass()) {
+                    case STRING: {
+                        _FString *existing   = (_FString*)varValue,
+                                 *new_string = ((_FString*)theP);
+                        
+                        if (new_string->empty()) {
+                            existing->SetData (kEmptyString);
+                        } else {
+                            existing->SetData (((_FString*)theP)->get_str());
+                        }
+                        return;
+                    }
+                        
+                }
+            }
+                        
             DeleteObject (varValue);
             varValue=nil;
         }
       
-        /*if (valueClass & (TREE)) {
-            variablePtrs.list_data[theIndex] = (long)(((_TheTree*)theP)->makeDynamicCopy(GetName()));
-            DeleteObject(this);
-        } else*/ {
-            if (dup) {
-                varValue = (HBLObjectRef)theP->makeDynamic();
-            } else {
-                varValue = theP;
-            }
+        
+        if (dup) {
+            varValue = (HBLObjectRef)theP->makeDynamic();
+        } else {
+            varValue = theP;
         }
     }
 }
@@ -442,6 +508,7 @@ void  _Variable::SetNumericValue (hyFloat v) // set the value of the var to a nu
     varFlags |= HY_VARIABLE_CHANGED;
     theValue = v;
 
+    
     if (theValue<lowerBound || theValue>upperBound) {
         if (theValue<=lowerBound+1e-50) {
             theValue = lowerBound;
@@ -453,11 +520,9 @@ void  _Variable::SetNumericValue (hyFloat v) // set the value of the var to a nu
 
 //__________________________________________________________________________________
 
-void  _Variable::CheckAndSet (hyFloat c, bool oob) // set the value of the var
+void  _Variable::CheckAndSet (hyFloat c, bool oob, _AVLList * keep_track_of_changes) // set the value of the var
 {
     //hasBeenChanged = true;
-    varFlags &= HY_VARIABLE_SET;
-    varFlags |= HY_VARIABLE_CHANGED;
     hyFloat l = lowerBound+1.0e-30,
                u = upperBound-1.0e-30;
     if (c<l || c>u ) {
@@ -473,12 +538,24 @@ void  _Variable::CheckAndSet (hyFloat c, bool oob) // set the value of the var
     } else {
         theValue = c;
     }
+    
+    varFlags &= HY_VARIABLE_SET;
+    varFlags |= HY_VARIABLE_CHANGED;
 
-    if (varValue) {
-        DeleteObject (varValue);
+    if (keep_track_of_changes) {
+        keep_track_of_changes->InsertNumber (get_index());
     }
 
+    if (varValue) {
+        if (varValue->ObjectClass() == NUMBER && varValue->SingleReference()) {
+            ((_Constant*)varValue)->theValue = theValue;
+            return;
+        }
+        DeleteObject (varValue);
+    }
+    
     varValue =  new _Constant(theValue);
+    
 }
 
 //__________________________________________________________________________________
@@ -499,9 +576,9 @@ void    _Variable::EnsureTheValueIsInBounds (void)
     if (ObjectClass () == NUMBER && IsIndependent()) {
         _Constant*   myValue = (_Constant*) Compute();
         if (myValue->Value() < lowerBound) {
-            SetValue (new _Constant (lowerBound),false);
+            SetValue (new _Constant (lowerBound),false,true,NULL);
         } else if (myValue->Value() > upperBound) {
-            SetValue (new _Constant (upperBound),false);
+            SetValue (new _Constant (upperBound),false,true,NULL);
         }
     }
 }
@@ -512,7 +589,7 @@ void    _Variable::ClearConstraints (void)
 {
     if (IsCategory ()) {
         _Variable newVar (*GetName(), IsGlobal());
-        newVar.SetValue ((HBLObjectRef)Compute()->makeDynamic(),false);
+        newVar.SetValue ((HBLObjectRef)Compute()->makeDynamic(),false,true,NULL);
         ReplaceVar ( &newVar);
         /*_Matrix * modelMatrix = (_Matrix*)LocateVar(modelMatrixIndices.list_data[1])->GetValue();
         for (long k=0; k<4; k++)
@@ -527,7 +604,7 @@ void    _Variable::ClearConstraints (void)
         //printf ("ClearConstraints %s %x\n", GetName()->get_str(), varFormula);
 
         if (!IsIndependent()) {
-            SetValue ((HBLObjectRef)Compute()->makeDynamic(),false);
+            SetValue ((HBLObjectRef)Compute()->makeDynamic(),false,true,NULL);
         }
         SetBounds (DEFAULTLOWERBOUND,DEFAULTUPPERBOUND);
     }
@@ -584,7 +661,7 @@ void  _Variable::SetFormula (_Formula& theF) {
     theF.ScanFForVariables (vA,true);
 
     if (vA.Find((BaseRefConst)theIndex)>=0) {
-        HandleApplicationError ((_String("Can't set variable ")&*GetName()&" to "&*((_String*)theF.toStr(kFormulaStringConversionNormal))&" because it would create a circular dependance."));
+        HandleApplicationError ((_String("Can't set variable ")&*GetName()&" to "&*((_String*)theF.toStr(kFormulaStringConversionNormal))&" because it would create a circular dependence."));
         if (&theF!=right_hand_side) {
             delete right_hand_side;
         }
@@ -683,11 +760,6 @@ bool  _Variable::HasChanged (bool ignoreCats) {
             return false;
         }
 
-        /*if (varFlags&HY_DEP_V_INSPECTED) {
-            return ignoreCats?(varFlags&HY_DEP_V_MODIFIED_CATS):(varFlags&HY_DEP_V_MODIFIED);
-        }*/
-        // SLKP 20170202: seems to be unused
-
        return  !varValue || varFormula->HasChanged(ignoreCats) ;
 
     } else {
@@ -729,6 +801,19 @@ _String const    _Variable::ContextFreeName(void) const {
     return *theName;
 }
 
+//__________________________________________________________________________________
+_StringBuffer&     _Variable::ContextFreeName(_StringBuffer & storage) const {
+    static const _String kDot (".");
+    
+    long location = theName->FindBackwards (kDot, 0L, kStringEnd);
+    if (location > 0L) {
+       return storage.AppendSubstring(*theName,location+1L,kStringEnd);
+    } else {
+        storage << *theName;
+    }
+    return storage;
+}
+
 //__________________________________________________________________________________
 _String const   _Variable::ParentObjectName(void) const {
     static const _String kDot (".");
