@@ -164,7 +164,7 @@ BaseRef _VariableContainer::toStr (unsigned long) {
 //__________________________________________________________________________________
 
 _VariableContainer::_VariableContainer (_String const & theName, _String theTmplt, _VariableContainer* theP) : iVariables(nil), dVariables(nil), gVariables(nil) {
-    InitializeVarCont (theName, theTmplt,theP);
+    _VariableContainer::InitializeVarCont (theName, theTmplt,theP);
 }
 
 //__________________________________________________________________________________
@@ -253,7 +253,6 @@ _Matrix* _VariableContainer::GetFreqMatrix (void) const  {
 void    _VariableContainer::ScanModelBasedVariables (_String const & fullName, _AVLListXL* varCache) {
     if (theModel!= HY_NO_MODEL) { // build the matrix variables
         _SimpleList       mVars;
-        _String           varName;
         
         {
             
@@ -286,8 +285,11 @@ void    _VariableContainer::ScanModelBasedVariables (_String const & fullName, _
             if (aVar->IsGlobal()) {
                 PushGlobalVariable(aVar->get_index());
             } else {
-                varName = fullName&'.'&aVar->ContextFreeName();
-                _Variable * spawnedVar = CheckReceptacle(&varName, kEmptyString, false, false);
+                _StringBuffer var_name (fullName.length() + 1UL + aVar->GetName()->length());
+                var_name << fullName << '.';
+                aVar->ContextFreeName(var_name);
+                //_String           var_name = fullName&'.'&aVar->ContextFreeName();
+                _Variable * spawnedVar = CheckReceptacle(&var_name, kEmptyString, false, false);
                 spawnedVar->SetBounds (aVar->GetLowerBound(), aVar->GetUpperBound());
 
                 if (aVar->IsIndependent()) {
@@ -348,7 +350,7 @@ void _VariableContainer::ScanAndAttachVariables (void) {
     long f = variableNames.Find (theName,travcache);
     
     if (f >= 0L) {
-        _String theNameAndADot = *theName & '.';
+        _StringBuffer theNameAndADot = (_StringBuffer (theName->length() + 1L) << *theName << '.');
 
         for (f = variableNames.Next (f, travcache); f>=0; f = variableNames.Next (f, travcache)) {
             var = FetchVar (f);
@@ -450,24 +452,23 @@ _Variable* _VariableContainer::GetIthParameter (long index) const {
 //__________________________________________________________________________________
 
 bool _VariableContainer::NeedToExponentiate (bool ignoreCats) const {
-    if (HY_VC_NO_CHECK&varFlags) {
-        return false;
-    }
-
-    auto has_changed = [=] (long var_index, long ref_index, unsigned long) -> bool {
-        if (ref_index >= 0L) {
+    if ((HY_VC_NO_CHECK&varFlags) == 0) {
+        auto has_changed = [=] (long var_index, long ref_index, unsigned long) -> bool {
+            if (ref_index >= 0L) {
+                return LocateVar (var_index) -> HasChanged (ignoreCats);
+            }
+            return false;
+        };
+        auto has_changed_global = [=] (long var_index, unsigned long) -> bool {
             return LocateVar (var_index) -> HasChanged (ignoreCats);
-        }
-        return false;
-    };
-    auto has_changed_global = [=] (long var_index, unsigned long) -> bool {
-        return LocateVar (var_index) -> HasChanged (ignoreCats);
-    };
+        };
 
-    
-    return AnyLocalVariable (iVariables, has_changed) ||
-           gVariables && gVariables->Any(has_changed_global) ||
-           AnyLocalVariable (dVariables, has_changed);
+        
+        return AnyLocalVariable (iVariables, has_changed) ||
+               gVariables && gVariables->Any(has_changed_global) ||
+               AnyLocalVariable (dVariables, has_changed);
+    }
+    return false;
     
 }
 
@@ -571,7 +572,7 @@ bool      _VariableContainer::RemoveDependance (long varIndex) {
     if (dVariables) {
         long array_index = dVariables->FindStepping(varIndex,2L);
 
-        if (array_index >= 0) {
+        if (array_index >= 0L) {
 
             InsertVariableInSortedList(iVariables,
                                        *LocateVar (dVariables->list_data[array_index])->GetName(),
@@ -583,6 +584,33 @@ bool      _VariableContainer::RemoveDependance (long varIndex) {
     return true;
 }
 
+//__________________________________________________________________________________
+void      _VariableContainer::RemoveDependance (_AVLList const & list) {
+    if (dVariables) {
+    
+        _SimpleList removed;
+        
+        ForEachLocalVariable(dVariables, [&list, &removed, this] (long self, long template_var, unsigned long array_idx) -> void {
+            if (list.FindLong(self) >= 0L) {
+                InsertVariableInSortedList(this->iVariables,
+                                           *LocateVar (self)->GetName(),
+                                           self,
+                                           template_var);
+                removed << array_idx;
+                
+            }
+        });
+        
+        removed.Sort();
+        removed.Flip();
+        
+        removed.Each([this] (long idx, unsigned long) -> void {
+            this->RemoveLocalVariable (this->dVariables, idx);
+        });
+        
+    }
+}
+
 //__________________________________________________________________________________
 long      _VariableContainer::CheckAndAddUserExpression (_String& parameter_name, long start_with) {
     _String localized_name = WrapInNamespace (parameter_name, theName),
@@ -650,22 +678,22 @@ void      _VariableContainer::CopyMatrixParameters (_VariableContainer* source,
             _SimpleList the_mapping;
             target_vars.Map (source_vars, the_mapping);
             the_mapping.Each ([=] (long source_var, unsigned long index) -> void {
-                if (source_var >= 0UL) {
+                if (source_var >= 0L) {
                     long which_idx = model_vars_in_source.list_data[source_var];
                     which_idx = which_idx >= 0 ? source->iVariables->get (which_idx) : source->dVariables->get (-which_idx-2L);
-                    LocateVar (iVariables->get (model_vars_in_target.get(index)))->SetValue (LocateVar (which_idx)->Compute());
+                    LocateVar (iVariables->get (model_vars_in_target.get(index)))->SetValue (LocateVar (which_idx)->Compute(),true,true,NULL);
                 }
             });
             
         } else {
             if (source->iVariables) {
                 for (unsigned long i=0UL; i<iVariables->lLength && i< source->iVariables->lLength; i+=2UL) {
-                    LocateVar (iVariables->get(i))->SetValue(LocateVar (source->iVariables->get(i))->Compute());
+                    LocateVar (iVariables->get(i))->SetValue(LocateVar (source->iVariables->get(i))->Compute(),true,true,NULL);
                 }
             }
         }
     }
-    SetValue (source->Compute());
+    SetValue (source->Compute(),true,true,NULL);
 }
 
 //__________________________________________________________________________________
@@ -825,18 +853,18 @@ void      _VariableContainer::ClearConstraints(void) {
     }
 }
 
+//#define _UBER_VERBOSE_MX_UPDATE_DUMP
+
 //__________________________________________________________________________________
 
 void      _VariableContainer::CopyModelParameterValue (long var_idx, long ref_idx, unsigned long) {
     if (ref_idx >= 0) {
         _Variable * model_var = LocateVar (ref_idx);
         if (model_var -> IsIndependent()) {
-            model_var->SetValue (LocateVar (var_idx)->Compute());
+            model_var->SetValue (LocateVar (var_idx)->Compute(),true,true,NULL);
             
 #ifdef _UBER_VERBOSE_MX_UPDATE_DUMP
-            if (1 || l == _UBER_VERBOSE_MX_UPDATE_DUMP_LF_EVAL) {
                 fprintf (stderr, "[_CalcNode::RecomputeMatrix] Node %s, var %s, value = %15.12g\n", LocateVar (var_idx)->GetName()->get_str(), model_var->GetName()->get_str(), model_var->Compute()->Value());
-            }
 #endif
         }
     }
@@ -883,6 +911,7 @@ void _VariableContainer::MarkDone (void) {
     }
 }
 
+
 //__________________________________________________________________________________
 
 void _VariableContainer::MatchParametersToList (_List& suffixes, bool doAll, bool indOnly) {
@@ -955,14 +984,22 @@ bool _VariableContainer::IsConstant (void) {
 
 //__________________________________________________________________________________
 
-void _VariableContainer::ScanContainerForVariables (_AVLList& l,_AVLList& l2, _AVLListX * tagger, long weight) {
+void _VariableContainer::ScanContainerForVariables (_AVLList& l,_AVLList& l2, _AVLListX * tagger, long weight, _AVLListX * map_variables_to_nodes, long track_node) {
     
     //printf ("_VariableContainer::ScanContainerForVariable %x\n", this);
     ForEachLocalVariable(iVariables, [&] (long var_idx, long ref_idx, unsigned long) -> void {
-        l.Insert((BaseRef)var_idx);
+        long insert_location = l.Insert((BaseRef)var_idx);
         if (tagger) {
             tagger->UpdateValue ((BaseRef)var_idx, weight, 0);
         }
+        if (map_variables_to_nodes) {
+            //printf ("%ld (%s) -> %ld\n", var_idx, LocateVar (var_idx)->GetName()->get_str(), insert_location);
+            if (insert_location >= 0) { // was inserted
+                map_variables_to_nodes->Insert ((BaseRef)var_idx, track_node, false, false);
+            } else { // was already there
+                map_variables_to_nodes->UpdateValue((BaseRef)var_idx, -1, 1);
+            }
+        }
     });
     
     if (dVariables) {
@@ -974,14 +1011,21 @@ void _VariableContainer::ScanContainerForVariables (_AVLList& l,_AVLList& l2, _A
             LocateVar (var_idx)->ScanForVariables(ta, true, tagger, weight);
         });
 
-        ta.ReorderList();
+        //ta.ReorderList();
         temp.Each([&] (long var_index, unsigned long) -> void {
             _Variable * v = LocateVar(var_index);
             if (!v->IsGlobal() && v->IsIndependent()) {
-                l.Insert ((BaseRef)var_index);
+                long insert_location = l.Insert ((BaseRef)var_index);
                 if (tagger) {
                     tagger->UpdateValue ((BaseRef)var_index, weight, 0);
                 }
+                if (map_variables_to_nodes) {
+                    if (insert_location >= 0) { // was inserted
+                        map_variables_to_nodes->Insert ((BaseRef)var_index, track_node, false, false);
+                    } else { // was already there
+                        map_variables_to_nodes->UpdateValue((BaseRef)var_index, -1, 1);
+                    }
+                }
             }
         });
     }
@@ -1009,10 +1053,9 @@ void _VariableContainer::GetListOfModelParameters (_List& rec) {
 
 void _VariableContainer::ScanForGVariables (_AVLList& independent,_AVLList& dependent, _AVLListX* tagger, long weight) const {
     
-    
     auto insert_g_var = [&] (_Variable *v, long var_idx) -> void {
         if (v->IsIndependent()) {
-            independent.Insert ((BaseRef)var_idx);
+            long insert_location = independent.Insert ((BaseRef)var_idx);
             if (tagger) {
                 tagger->UpdateValue((BaseRef)var_idx, weight, 0);
             }
@@ -1035,7 +1078,7 @@ void _VariableContainer::ScanForGVariables (_AVLList& independent,_AVLList& depe
         ForEachLocalVariable(dVariables, [&] (long var_idx, long ref_idx, unsigned long) -> void {
             LocateVar (var_idx)->ScanForVariables(al, true);
         });
-        al.ReorderList();
+        //al.ReorderList();
         var_list.Each ([&] (long var_idx, unsigned long) -> void {
             _Variable * v = LocateVar(var_idx);
             if (v->IsGlobal()) {
